import os 

from subprocess import *

from .popenwrapper import Popen

# Static class that allows the type of a file to be checked.
class FileType(object):
  # Provides int -> str map
  revMap = { }

  @classmethod
  def getFileType(cls, fileName):
      # This is a hacky way of determining
      # the type of file we are looking at.
      # Maybe we should use python-magic instead?

      fileP = Popen(['file',os.path.realpath(fileName)], stdout=PIPE)
      output = fileP.communicate()[0]
      output = output.decode()
      if 'ELF' in output and 'executable' in output:
          return cls.ELF_EXECUTABLE
      if 'Mach-O' in output and 'executable' in output:
          return cls.MACH_EXECUTABLE
      elif 'ELF' in output and 'shared' in output:
          return cls.ELF_SHARED
      elif 'Mach-O' in output and 'dynamically linked shared' in output:
          return cls.MACH_SHARED
      elif 'current ar archive' in output:
          return cls.ARCHIVE
      elif 'ELF' in output and 'relocatable' in output:
          return cls.ELF_OBJECT
      elif 'Mach-O' in output and 'object' in output:
          return cls.MACH_OBJECT
      else:
          return cls.UNKNOWN

  @classmethod
  def init(cls):
      for (index, name) in enumerate(('UNKNOWN',
                                      'ELF_EXECUTABLE',
                                      'ELF_OBJECT',
                                      'ELF_SHARED',
                                      'MACH_EXECUTABLE',
                                      'MACH_OBJECT',
                                      'MACH_SHARED',
                                      'ARCHIVE')):
          setattr(cls, name, index)
          cls.revMap[index] = name

# Initialise FileType static class
FileType.init()



# Freeping Creaturism:
#
# this is the all important version number used by pip.
#
#
"""
Version History:

1.0.0   - 8/2/2016 initial birth as a pip package.

1.0.1   - 8/2/2016 the rst gets a make over, and doc strings 
          became more pervasive.

1.0.2   - 8/4/2016 dragonegg issues. trying to include a polite 'as' wrapper
          (i.e. not a  console_script called as).

1.0.3   - 8/4/2016 travis build fixes.

1.0.4   - 8/4/2016 travis build fixes, and exception handling fixes.

1.0.5   - 8/4/2016 exit value was upsetting travis.

1.0.6   - 8/9/2016 exit codes preserved; important for configure scripts like musl libc.

1.0.7   - 8/9/2016 logical restructuring; smaller bites.

1.0.8   - 8/9/2016 test and doc subdirectories are no longer included.

1.0.9   - 8/25/2016  Python 3.0 import fixes (Will Dietz)

"""
wllvm_version='1.0.9'




#!/usr/bin/env python
"""This is a wrapper around the real compiler.

It first invokes a real compiler to generate
an object file.  Then it invokes a bitcode
compiler to generate a parallel bitcode file.
It records the location of the bitcode in an
ELF section of the object file so that it can be
found later after all of the objects are
linked into a library or executable.
"""

import sys, os

from .utils import getBuilder, buildObject, buildAndAttachBitcode, logging

from .logconfig import *

_logger = logging.getLogger(__name__)

def main():
    try:
        rc = 1
        cmd = list(sys.argv)
        cmd = cmd[1:]
        builder = getBuilder(cmd, False)
        rc = buildObject(builder)
        if rc == 0 and not os.environ.get('WLLVM_CONFIGURE_ONLY', False):
            buildAndAttachBitcode(builder)
    except Exception as e:
        _logger.debug('wllvm: exception case: {0}\n'.format(e))
        pass
    return rc


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
"""This does some simple sanity checks on the configuration.

It attempts to print informative results of that check.
Hopefully never dumping a python stack trace.

"""

import sys

from .checker import Checker

def main():
    return Checker().check()


if __name__ == '__main__':
    sys.exit(main())



import os
import sys
import subprocess as sp

from .utils import llvmCompilerPathEnv

from .popenwrapper import Popen

from .utils import elfSectionName
from .utils import darwinSegmentName
from .utils import darwinSectionName

from .filetype import FileType

from .logconfig import *

import logging
import pprint
import tempfile
import shutil
import argparse

"""
(Fix: 2016/02/16: __LLVM is now used by MacOS's ld so we changed the segment name to __WLLVM).

"""



# Python 2 does not have exceptions automatically
# imported whereas python 3 does. Handle this
try:
    dir(UnicodeDecodeError)
except NameError:
  import exceptions

bitCodeArchiveExtension='bca'
moduleExtension='bc'

def getSectionSizeAndOffset(sectionName, filename):
    """Returns the size and offset of the section, both in bytes.

    Use objdump on the provided binary; parse out the fields
    to find the given section.  Parses the output,and
    extracts thesize and offset of that section (in bytes).
    """
    objdumpCmd = ['objdump', '-h', '-w', filename]
    objdumpProc = Popen(objdumpCmd, stdout=sp.PIPE)

    objdumpOutput = objdumpProc.communicate()[0]
    if objdumpProc.returncode != 0:
        logging.error('Could not dump %s' % filename)
        sys.exit(-1)

    for line in [l.decode('utf-8') for l in objdumpOutput.splitlines()] :
        fields = line.split()
        if len(fields) <= 7:
            continue
        if fields[1] != sectionName:
            continue
        try:
            idx = int(fields[0])
            size = int(fields[2], 16)
            offset = int(fields[5], 16)
            return (size, offset)
        except ValueError:
            continue

    # The needed section could not be found
    logging.warning('Could not find "{0}" ELF section in "{1}", so skipping this entry.'.format(sectionName,filename))
    return None

def getSectionContent(size, offset, filename):
    """Reads the entire content of an ELF section into a string."""
    with open(filename, mode='rb') as f:
        f.seek(offset)
        d = ''
        try:
            c = f.read(size)
            d = c.decode('utf-8')
        except UnicodeDecodeError:
            logging.error('Failed to read section containing:')
            print(c)
            raise
        # The linker pads sections with null bytes; our real data
        # cannot have null bytes because it is just text.  Discard
        # nulls.
        return d.replace('\0', '')

def extract_section_darwin(inputFile):
    """Extracts the section as a string, the darwin version.

    Uses otool to extract the section, then processes it
    to a usable state.
    """

    otoolCmd  = ['otool', '-X', '-s', darwinSegmentName, darwinSectionName, inputFile]
    otoolProc = Popen(otoolCmd, stdout=sp.PIPE)

    otoolOutput = otoolProc.communicate()[0]
    if otoolProc.returncode != 0:
        logging.error('otool failed on %s' % inputFile)
        sys.exit(-1)

    lines = otoolOutput.splitlines()
    octets = []
    for line in lines:
        (_, octetline) = line.split('\t')
        octets.extend(octetline.split())
    octets = ''.join(octets)
    contents = octets.decode('hex').splitlines()
    if not contents:
        logging.error('{0} contained no {1} segment'.format(inputFile, darwinSegmentName))
    return contents

def extract_section_linux(inputFile):
    """Extracts the section as a string, the *nix version."""
    val = getSectionSizeAndOffset(elfSectionName, inputFile)
    if val is None:
        return []
    (sectionSize, sectionOffset) = val
    content = getSectionContent(sectionSize, sectionOffset, inputFile)
    contents = content.split('\n')
    if not contents:
        logging.error('{0} contained no {1} section is empty'.format(inputFile, elfSectionName))
    return contents


def linkFiles(pArgs, fileNames):
    linkCmd = [ pArgs.llvmLinker, '-v' ] if pArgs.verboseFlag else [ pArgs.llvmLinker ]

    linkCmd.extend(['-o', pArgs.outputFile ])

    linkCmd.extend([x for x in fileNames if x != ''])
    logging.info('Writing output to {0}'.format(pArgs.outputFile))
    try:
        linkProc = Popen(linkCmd)
    except OSError as e:
        if e.errno == 2:
            errorMsg = 'Your llvm-link does not seem to be easy to find.\nEither install it or use the -l llvmLinker option.'
        else:
            errorMsg = 'OS error({0}): {1}'.format(e.errno, e.strerror)
        logging.error(errorMsg)
        raise Exception(errorMsg)

    else:
        exitCode = linkProc.wait()
        logging.info('{0} returned {1}'.format(pArgs.llvmLinker, str(exitCode)))
    return exitCode


def archiveFiles(pArgs, fileNames):
    retCode = 0
    # We do not want full paths in the archive so we need to chdir into each
    # bitcode's folder. Handle this by calling llvm-ar once for all bitcode
    # files in the same directory

    # Map of directory names to list of bitcode files in that directory
    dirToBCMap = {}
    for bitCodeFile in fileNames:
        dirName = os.path.dirname(bitCodeFile)
        basename = os.path.basename(bitCodeFile)
        if dirName in dirToBCMap:
            dirToBCMap[dirName].append(basename)
        else:
            dirToBCMap[dirName] = [ basename ]

    logging.debug('Built up directory to bitcode file list map:\n{0}'.format(
                   pprint.pformat(dirToBCMap)))

    for (dirname, bcList) in dirToBCMap.items():
        logging.debug('Changing directory to "{0}"'.format(dirname))
        os.chdir(dirname)
        larCmd = [pArgs.llvmArchiver, 'rs', pArgs.outputFile ] + bcList
        larProc = Popen(larCmd)
        retCode = larProc.wait()
        if retCode != 0:
          logging.error('Failed to execute:\n{0}'.format(pprint.pformat(larCmd)))
          break

    if retCode == 0:
        logging.info('Generated LLVM bitcode archive {0}'.format(pArgs.outputFile))
    else:
        logging.error('Failed to generate LLVM bitcode archive')

    return retCode


def handleExecutable(pArgs):

    fileNames = pArgs.extractor(pArgs.inputFile)

    if not fileNames:
        return 1

    if pArgs.manifestFlag:
        manifestFile = '{0}.llvm.manifest'.format(pArgs.inputFile)
        with open(manifestFile, 'w') as output:
            for f in fileNames:
                output.write('{0}\n'.format(f))

    if pArgs.outputFile == None:
        pArgs.outputFile = pArgs.inputFile + '.' + moduleExtension

    return linkFiles(pArgs, fileNames)




def handleArchive(pArgs):

    originalDir = os.getcwd() # This will be the destination

    pArgs.arCmd.append(pArgs.inputFile)

    # Make temporary directory to extract objects to
    tempDir = ''
    bitCodeFiles = [ ]
    retCode=0
    try:
        tempDir = tempfile.mkdtemp(suffix='wllvm')
        os.chdir(tempDir)

        # Extract objects from archive
        try:
            arP = Popen(pArgs.arCmd)
        except OSError as e:
            if e.errno == 2:
                errorMsg = 'Your ar does not seem to be easy to find.\n'
            else:
                errorMsg = 'OS error({0}): {1}'.format(e.errno, e.strerror)
            logging.error(errorMsg)
            raise Exception(errorMsg)

        arPE = arP.wait()

        if arPE != 0:
            errorMsg = 'Failed to execute archiver with command {0}'.format(pArgs.arCmd)
            logging.error(errorMsg)
            raise Exception(errorMsg)

        # Iterate over objects and examine their bitcode inserts
        for (root, dirs, files) in os.walk(tempDir):
           logging.debug('Exploring "{0}"'.format(root))
           for f in files:
               fPath = os.path.join(root, f)
               if FileType.getFileType(fPath) == pArgs.fileType:

                   # Extract bitcode locations from object
                   contents = pArgs.extractor(fPath)

                   for bcFile in contents:
                       if bcFile != '':
                           if not os.path.exists(bcFile):
                               logging.warning('{0} lists bitcode library "{1}" but it could not be found'.format(f, bcFile))
                           else:
                               bitCodeFiles.append(bcFile)
               else:
                   logging.info('Ignoring file "{0}" in archive'.format(f))

        logging.info('Found the following bitcode file names to build bitcode archive:\n{0}'.format(
            pprint.pformat(bitCodeFiles)))

    finally:
        # Delete the temporary folder
        logging.debug('Deleting temporary folder "{0}"'.format(tempDir))
        shutil.rmtree(tempDir)

    #write the manifest file if asked for
    if pArgs.manifestFlag:
        manifestFile = '{0}.llvm.manifest'.format(pArgs.inputFile)
        with open(manifestFile, 'w') as output:
            for f in bitCodeFiles:
                output.write('{0}\n'.format(f))

    # Build bitcode archive
    os.chdir(originalDir)

    return buildArchive(pArgs, bitCodeFiles)

def buildArchive(pArgs, bitCodeFiles):

    if pArgs.bitcodeModuleFlag:

        # Pick output file path if outputFile not set
        if pArgs.outputFile == None:
            pArgs.outputFile = pArgs.inputFile
            pArgs.outputFile += '.' + moduleExtension

        logging.info('Writing output to {0}'.format(pArgs.outputFile))

        return linkFiles(pArgs, bitCodeFiles)

    else:

        # Pick output file path if outputFile not set
        if pArgs.outputFile == None:
            if pArgs.inputFile.endswith('.a'):
                # Strip off .a suffix
                pArgs.outputFile = pArgs.inputFile[:-2]
                pArgs.outputFile += '.' + bitCodeArchiveExtension

        logging.info('Writing output to {0}'.format(pArgs.outputFile))

        return archiveFiles(pArgs, bitCodeFiles)


class ExtractedArgs:

    fileType = None

    extractor = None

    arCmd = None


def extract_bc_args(args):

    # do we need a path in front?
    llvmToolPrefix = os.getenv(llvmCompilerPathEnv)
    if not llvmToolPrefix:
        llvmToolPrefix = ''

    # is our linker called something different?
    llvmLinkerName = os.getenv('LLVM_LINK_NAME')
    if not llvmLinkerName:
        llvmLinkerName = 'llvm-link'
    llvmLinker = os.path.join(llvmToolPrefix, llvmLinkerName)

    # is our archiver called something different?
    llvmArchiverName = os.getenv('LLVM_AR_NAME')
    if not llvmArchiverName:
        llvmArchiverName = 'llvm-ar'
    llvmArchiver = os.path.join(llvmToolPrefix, llvmArchiverName)

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(dest='inputFile',
                        help='A binary produced by wllvm/wllvm++')
    parser.add_argument('--linker','-l',
                        dest='llvmLinker',
                        help='The LLVM bitcode linker to use. Default "%(default)s"',
                        default=llvmLinker)
    parser.add_argument('--archiver','-a',
                        dest='llvmArchiver',
                        help='The LLVM bitcode archiver to use. Default "%(default)s"',
                        default=llvmArchiver)
    parser.add_argument('--verbose','-v',
                        dest='verboseFlag',
                        help='Call the external procedures in verbose mode.',
                        action="store_true")
    parser.add_argument('--manifest', '-m',
                        dest='manifestFlag',
                        help='Write a manifest file listing all the .bc files used.',
                        action='store_true')
    parser.add_argument('--bitcode', '-b',
                        dest='bitcodeModuleFlag',
                        help='Extract a bitcode module rather than an archive. ' +
                        'Only useful when extracting from an archive.',
                        action='store_true')
    parser.add_argument('--output','-o',
                        dest='outputFile',
                        help='The output file. Defaults to a file in the same directory ' +
                             'as the input with the same name as the input but with an ' +
                             'added file extension (.'+ moduleExtension + ' for bitcode '+
                             'modules and .' + bitCodeArchiveExtension +' for bitcode archives)',
                        default=None)
    pArgs = parser.parse_args(namespace=ExtractedArgs())


    # Check file exists
    if not os.path.exists(pArgs.inputFile):
        logging.error('File "{0}" does not exist.'.format(pArgs.inputFile))
        return (False, None)

    pArgs.inputFile = os.path.abspath(pArgs.inputFile)


    # Check output destitionation if set
    outputFile = pArgs.outputFile
    if outputFile != None:
      # Get Absolute output path
      outputFile = os.path.abspath(outputFile)
      if not os.path.exists(os.path.dirname(outputFile)):
        logging.error('Output directory "{0}" does not exist.'.format(os.path.dirname(outputFile)))
        return (False, None)

    pArgs.output = outputFile

    return (True, pArgs)


def extraction(args):

    (success, pArgs) = extract_bc_args(args)

    if not success:
        return 1

    if ( sys.platform.startswith('freebsd') or  sys.platform.startswith('linux') ):

        process_file_unix(pArgs)

    elif sys.platform.startswith('darwin'):

        process_file_darwin(pArgs)

    else:
        #iam: do we work on anything else?
        logging.error('Unsupported or unrecognized platform: {0}'.format(sys.platform))
        return 1


def process_file_unix(pArgs):

    ft = FileType.getFileType(pArgs.inputFile)
    logging.debug('Detected file type is {0}'.format(FileType.revMap[ft]))

    pArgs.arCmd   =  ['ar', 'xv'] if pArgs.verboseFlag else ['ar', 'x']
    pArgs.extractor = extract_section_linux
    pArgs.fileType = FileType.ELF_OBJECT

    if ft == FileType.ELF_EXECUTABLE or ft == FileType.ELF_SHARED or ft == FileType.ELF_OBJECT:
        logging.info('Generating LLVM Bitcode module')
        return handleExecutable(pArgs)
    elif ft == FileType.ARCHIVE:
        if pArgs.bitcodeModuleFlag:
            logging.info('Generating LLVM Bitcode module from an archive')
        else:
            logging.info('Generating LLVM Bitcode archive from an archive')
        return handleArchive(pArgs)
    else:
        logging.error('File "{0}" of type {1} cannot be used'.format(pArgs.inputFile, FileType.revMap[ft]))
        return 1



def process_file_darwin(pArgs):

    ft = FileType.getFileType(pArgs.inputFile)
    logging.debug('Detected file type is {0}'.format(FileType.revMap[ft]))

    pArgs.arCmd   =  ['ar', '-x', '-v'] if pArgs.verboseFlag else ['ar', '-x']
    pArgs.extractor = extract_section_darwin
    pArgs.fileType =  FileType.MACH_OBJECT

    if ft == FileType.MACH_EXECUTABLE or ft == FileType.MACH_SHARED or ft == FileType.MACH_OBJECT:
        logging.info('Generating LLVM Bitcode module')
        return handleExecutable(pArgs)
    elif ft == FileType.ARCHIVE:
        if pArgs.bitcodeModuleFlag:
            logging.info('Generating LLVM Bitcode module from an archive')
        else:
            logging.info('Generating LLVM Bitcode archive from an archive')
        return handleArchive(pArgs)
    else:
        logging.error('File "{0}" of type {1} cannot be used'.format(pArgs.inputFile, FileType.revMap[ft]))
        return 1




#!/usr/bin/env python
"""This is a wrapper around the real compiler.

It first invokes a real compiler to generate an object file.  Then it
invokes a bitcode compiler to generate a parallel bitcode file.  It
records the location of the bitcode in an ELF section of the object
file so that it can be found later after all of the objects are linked
into a library or executable.
"""
import sys, os

from .utils import getBuilder, buildObject, buildAndAttachBitcode, logging

from .logconfig import *

_logger = logging.getLogger(__name__)

def main():
    try:
        rc = 1
        cmd = list(sys.argv)
        cmd = cmd[1:]
        builder = getBuilder(cmd, True)
        rc = buildObject(builder)


        if rc == 0 and not os.environ.get('WLLVM_CONFIGURE_ONLY', False):
            buildAndAttachBitcode(builder)
    except Exception as e:
        _logger.debug('wllvm++: exception case: {0}\n'.format(e))
        pass
    return rc


if __name__ == '__main__':
    sys.exit(main())




import sys
import os
import subprocess as sp
import errno
"""
Module support for the wllvm-sanity-checker tool.

The wllvm-sanity-checker tool examines the users
environment to see if it makes sense from the
wllvm point of view. Useful first step in trying to
debug a failure.
"""

explain_LLVM_COMPILER = """

The environment variable 'LLVM_COMPILER' is a switch. It should either
be set to 'clang' or 'dragonegg'. Anything else will cause an error.

"""

explain_LLVM_DRAGONEGG_PLUGIN = """

You need to set the environment variable LLVM_DRAGONEGG_PLUGIN to the
full path to your dragonegg plugin. Thanks.

"""

explain_LLVM_CC_NAME = """

If your clang compiler is not called clang, but something else, then
you will need to set the environment variable LLVM_CC_NAME to the
appropriate string. For example if your clang is called clang-3.5 then
LLVM_CC_NAME should be set to clang-3.5.

"""

explain_LLVM_CXX_NAME = """

If your clang++ compiler is not called clang++, but something else,
then you will need to set the environment variable LLVM_CXX_NAME to
the appropriate string. For example if your clang++ is called ++clang
then LLVM_CC_NAME should be set to ++clang.

"""

explain_LLVM_COMPILER_PATH = """

Your compiler should either be in your PATH, or else located where the
environment variable LLVM_COMPILER_PATH indicates. It can also be used
to indicate the directory that contains the other LLVM tools such as
llvm-link, and llvm-ar.

"""

explain_LLVM_LINK_NAME = """

If your llvm linker is not called llvm-link, but something else, then
you will need to set the environment variable LLVM_LINK_NAME to the
appropriate string. For example if your llvm-link is called llvm-link-3.5 then
LLVM_LINK_NAME should be set to llvm-link-3.5.

"""

explain_LLVM_AR_NAME = """

If your llvm archiver is not called llvm-ar, but something else,
then you will need to set the environment variable LLVM_AR_NAME to
the appropriate string. For example if your llvm-ar is called llvm-ar-3.5
then LLVM_AR_NAME should be set to llvm-ar-3.5.

"""

class Checker(object):
    def __init__(self):
        path = os.getenv('LLVM_COMPILER_PATH')

        if path and path[-1] != os.path.sep:
            path = path + os.path.sep

        self.path = path if path else ''

    def check(self):
        """Performs the environmental sanity check.

        Performs the following checks in order:

        1. Check that the OS is supported.
        2. Checks that the compiler settings make sense.
        3. Checks that the needed LLVM utilities exists.
        """

        if not self.checkOS():
            print('I do not think we support your OS. Sorry.')
            return 1

        success = self.checkCompiler()

        if success:
            self.checkAuxiliaries()

        return 0 if success else 1



    def checkOS(self):
        """Returns True if we support the OS."""
        return (sys.platform.startswith('freebsd') or
                sys.platform.startswith('linux') or
                sys.platform.startswith('darwin'))


    def checkSwitch(self):
        """Checks the correctness of the LLVM_COMPILER env var."""
        compiler_type = os.getenv('LLVM_COMPILER')
        if compiler_type == 'clang':
            return (1, '\nGood, we are using clang.\n')
        elif compiler_type == 'dragonegg':
            return (2, '\nOK, we are using dragonegg.\n')
        else:
            return (0, explain_LLVM_COMPILER)


    def checkClang(self):
        """Checks for clang and clang++."""
        cc_name = os.getenv('LLVM_CC_NAME')
        cxx_name = os.getenv('LLVM_CXX_NAME')

        cc =  '{0}{1}'.format(self.path, cc_name if cc_name else 'clang')
        cxx = '{0}{1}'.format(self.path, cxx_name if cxx_name else 'clang++')

        return self.checkCompilers(cc, cxx)


    def checkDragonegg(self):
        """Checks for gcc, g++ and the dragonegg plugin."""
        if not self.checkDragoneggPlugin():
            return False

        pfx = ''
        if os.getenv('LLVM_GCC_PREFIX') is not None:
            pfx = os.getenv('LLVM_GCC_PREFIX')

        cc  = '{0}{1}gcc'.format(self.path, pfx)
        cxx = '{0}{1}g++'.format(self.path, pfx)

        return self.checkCompilers(cc, cxx)


    def checkDragoneggPlugin(self):
        """Checks for the dragonegg plugin."""
        plugin = os.getenv('LLVM_DRAGONEGG_PLUGIN')

        if not plugin:
            print(explain_LLVM_DRAGONEGG_PLUGIN)
            return False

        if os.path.isfile(plugin):
            try:
                open(plugin)
                pass
            except IOError as e:
                print("Unable to open {0}".format(plugin))
            else:
                return True
        else:
            print("Could not find {0}".format(plugin))
            return False


    def checkCompiler(self):
        """Determines the chosen compiler, and checks it."""
        (code, comment) = self.checkSwitch()

        if code == 0:
            print(comment)
            return False
        elif code == 1:
            print(comment)
            return self.checkClang()
        elif code == 2:
            print(comment)
            return self.checkDragonegg()
        else:
            print('Insane\n')
            return False



    def checkCompilers(self, cc, cxx):
        """Tests that the compilers actually exist."""
        (ccOk, ccVersion) = self.checkExecutable(cc)
        (cxxOk, cxxVersion) = self.checkExecutable(cxx)

        if not ccOk:
            print('The C compiler {0} was not found or not executable.\nBetter not try using wllvm!\n'.format(cc))
        else:
            print('The C compiler {0} is:\n\n{1}\n'.format(cc, extractLine(ccVersion, 0)))

        if not cxxOk:
            print('The CXX compiler {0} was not found or not executable.\nBetter not try using wllvm++!\n'.format(cxx))
        else:
            print('The C++ compiler {0} is:\n\n{1}\n'.format(cxx, extractLine(cxxVersion, 0)))

        if not ccOk or  not cxxOk:
            print(explain_LLVM_COMPILER_PATH)
            if not ccOk:
                print(explain_LLVM_CC_NAME)
            if not cxxOk:
                print(explain_LLVM_CXX_NAME)



        return ccOk or cxxOk


    def checkExecutable(self, exe, version_switch='-v'):
        """Checks that an executable exists, and is executable."""
        cmd = [exe, version_switch]
        try:
            compiler = sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.PIPE)
            output = compiler.communicate()
            compilerOutput = '{0}{1}'.format(output[0], output[1])
        except OSError as e:
            if e.errno == errno.EPERM:
                return (False, '{0} not executable'.format(exe))
            elif e.errno == errno.ENOENT:
                return (False, '{0} not found'.format(exe))
            else:
                return (False, '{0} not sure why, errno is {1}'.format(exe, e.errno))
        else:
            return (True, compilerOutput)



    def checkAuxiliaries(self):
        """Checks for the archiver and linker."""
        link_name = os.getenv('LLVM_LINK_NAME')
        ar_name = os.getenv('LLVM_AR_NAME')

        if not link_name:
            link_name = 'llvm-link'

        if not ar_name:
            ar_name = 'llvm-ar'

        link = '{0}{1}'.format(self.path,link_name) if self.path else link_name
        ar = '{0}{1}'.format(self.path,ar_name) if self.path else ar_name

        (linkOk, linkVersion) = self.checkExecutable(link, '-version')

        (arOk, arVersion) =  self.checkExecutable(ar, '-version')

        if not linkOk:
            print('The bitcode linker {0} was not found or not executable.\nBetter not try using extract-bc!\n'.format(link))
            print(explain_LLVM_LINK_NAME)
        else:
            print('The bitcode linker {0} is:\n\n{1}\n'.format(link, extractLine(linkVersion, 1)))

        if not arOk:
            print('The bitcode archiver {0} was not found or not executable.\nBetter not try using extract-bc!\n'.format(ar))
            print(explain_LLVM_AR_NAME)
        else:
            print('The bitcode archiver {0} is:\n\n{1}\n'.format(ar, extractLine(arVersion, 1)))



def extractLine(version, n):
        if not version:
            return version
        lines = version.split('\n')
        if n < len(lines):
            return lines[n]
        else:
            return lines[-1]
